/*
 *  Chris Adams 04-01-2016  - bumper-bot ver 1f
 * 
    Toy car with Realtek RX2(RX2B)chip  removed
    
    PWM ports connected to on board H-bridge

    using Arduino Pro-mini

    micro switches on front and rear bumpers
    small drive wheels added to reduce speed

    jumper on top of pro-mini bridges A4-A5

    extra AA battery in series with 4 AA batteries in car 
    connected to Vin
 
 */

#define RIGHT_REV      5
#define RIGHT_FWD      6  //
#define LEFT_FWD       10   //OK
#define LEFT_REV       11  //OK

#define stopped 0
#define forward 1
#define reverse 2
#define spinning 3

//output pins
#define LED 13

//input pins
#define frontBumper   2
#define backBumper    3

#define Jumper1   9
#define Jumper2   8

boolean LED_on=false;
boolean jumper = false;
boolean debug=0;

int hits=0;
int hit = 0;
int hit_stop=0;
int last_hit = 0;
int pass=0;
int spin=0;

int mode;
boolean started = false;

//interrupt
unsigned long currentMillis  = 0;
unsigned long hitMillis  = 0;


void setup() {

    Serial.begin(9600);
 
  

  

  pinMode(Jumper1, INPUT_PULLUP);
  pinMode(Jumper2, INPUT_PULLUP);

  
  if (digitalRead(Jumper1))  // turn on debug -jumper 9 to gnd
      debug = 0;
  else
      debug = 1;

  // jumper on top of pro-mini A4-A5
  pinMode(A4, INPUT); 
  digitalWrite(A5, HIGH); 
  if (analogRead(A4) > 1000)
    jumper = true;
  else
    jumper = false;

  

  pinMode(LEFT_REV, OUTPUT);
  digitalWrite(LEFT_REV, LOW);
  pinMode(RIGHT_REV, OUTPUT);
  digitalWrite(RIGHT_REV, LOW);


  pinMode(backBumper, INPUT_PULLUP);
  pinMode(frontBumper, INPUT_PULLUP);
  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);

   if(debug)
       Serial.print ("STOPPED-tap-to-start ");
   // start by tapping front bumper
   hitMillis=millis();    // time of LED blink
   while(digitalRead(frontBumper)){
          if (millis() - hitMillis > 330){
                 LED_on = !LED_on;      // blink while stopped
                 if (LED_on)
                   digitalWrite(LED, HIGH);
                 else
                   digitalWrite(LED, LOW);
                 hitMillis=millis();    // time of LED blink
          }
    }

    delay(2500);    //wait 3 seconds before moving

    digitalWrite(LED, HIGH);
   if(!debug){
    analogWrite(RIGHT_FWD, 75);
    analogWrite(LEFT_FWD, 75);
   }
    mode=forward;   

}


void loop() {
if(debug){

      Serial.print ("  hit=");
      Serial.print (hit);
      Serial.print ("  hits=");
      Serial.print (hits);
      Serial.print ("  hit_stop=");
      Serial.print (hit_stop);
      if (mode==1)
        Serial.print ("  forward");
      if (mode==2)
        Serial.print ("  reverse");
      if (mode==3)
        Serial.print ("  spinning");
      if (mode==0)
        Serial.print ("  stopped");
      Serial.print ("  mode=");
      Serial.println (mode);
}

  if (analogRead(A4) > 1000) {
    jumper = true;
  }
  else {
    jumper = false;
  }

  // we hit something going forward
  // go reverse now
  if (digitalRead(frontBumper) == LOW) {
     digitalWrite(RIGHT_FWD, LOW);    // stop motors
     digitalWrite(LEFT_FWD, LOW);
     digitalWrite(LED, LOW);
     delay(100);  // debounce
     hitMillis=millis();    // save time of hit
   if(!debug){  // turn off motors in debug mode
     analogWrite(RIGHT_REV, 75);
     analogWrite(LEFT_REV, 75);
   }
     delay(10);  // debounce
     mode=reverse;
     while(digitalRead(frontBumper) == LOW){    // wait for switch to release
     }     // wait till we move
     delay(12);   // debounce bumper
     while(digitalRead(frontBumper) == LOW){
      // wait some more till we move 
     }
     hit = frontBumper;
     ++hits;
     ++hit_stop;    // total hits
     
  // we hit something backing up
  // go forward now
  } else if (digitalRead(backBumper) == LOW) {  // 2a
     digitalWrite(RIGHT_REV, LOW);
     digitalWrite(LEFT_REV, LOW);
     digitalWrite(LED, HIGH);
     delay(70);  // debounce
     hitMillis=millis();    // save time of hit
   if(!debug){  // turn off motors in debug mode
     analogWrite(RIGHT_FWD, 75);
     analogWrite(LEFT_FWD, 75);
   }
     delay(10);  // debounce
     mode=forward; 
     while(digitalRead(backBumper) == LOW){ // wait for switch to release
     }     // wait till we move
     delay(12);   // debounce bumper
     while(digitalRead(backBumper) == LOW){
      // wait some more till we move 
     }
     hit = backBumper;
     ++hits;
     ++hit_stop;      // total hits
     
     
  // after 13 bumper hits and moving for 500ms
  // stop here and wait for front bumper to be tapped
  // jumper must be plugged in for 13 hit rule
  } else if ((millis() - hitMillis > 500) && hit_stop >= 15 && jumper) {
         digitalWrite(RIGHT_FWD, LOW);
         digitalWrite(LEFT_FWD, LOW);
         digitalWrite(RIGHT_REV, LOW);
         digitalWrite(LEFT_REV, LOW);
         mode=stopped;
         if(debug)
            Serial.print ("STOPPED-15hits ");
         // start by tapping front bumper
         hitMillis=millis();    // time of LED blink
         while(digitalRead(frontBumper)){
             if (millis() - hitMillis > 330){
                 LED_on = !LED_on;
                 if (LED_on)
                   digitalWrite(LED, HIGH);
                 else
                   digitalWrite(LED, LOW);
                 hitMillis=millis();    // time of LED blink
             }
         }
         digitalWrite(LED, HIGH);
         delay(2500);    //wait 3 seconds before moving
   if(!debug){  // turn off motors in debug mode
         analogWrite(RIGHT_FWD, 75);
         analogWrite(LEFT_FWD, 75);
   }
         mode=forward;   
         hits = 0;
         hit_stop=0;
     
     
  // after hitting something twice
  // we have changed directions and moved for 500ms now
  // stop here and spin for 500ms
  } else if ((millis() - hitMillis > 500) && hits >= 2) {
         digitalWrite(RIGHT_FWD, LOW);
         digitalWrite(LEFT_REV, LOW);
      if(!debug){  // turn off motors in debug mode
         analogWrite(RIGHT_REV, 175);
         analogWrite(LEFT_FWD, 175);
      }
         mode=spinning;
         if(debug)
            Serial.print ("SPINNING ");
         delay(2000);
         digitalWrite(RIGHT_FWD, LOW);
         digitalWrite(LEFT_FWD, LOW);
         digitalWrite(RIGHT_REV, LOW);
         digitalWrite(LEFT_REV, LOW);
         mode=stopped;
         if(debug)
            Serial.print ("STOPPED ");
         delay(100);
         digitalWrite(RIGHT_REV, LOW);
         digitalWrite(LEFT_REV, LOW);
      if(!debug){  // turn off motors in debug mode
         analogWrite(RIGHT_FWD, 75);
         analogWrite(LEFT_FWD, 75);
      }
         mode=forward;
         digitalWrite(LED, HIGH);
         hits = 0;

  } else if (millis() - hitMillis > 30000){ //stop after 30 seconds with no hits
          digitalWrite(RIGHT_FWD, LOW);
          digitalWrite(LEFT_FWD, LOW);
          digitalWrite(RIGHT_REV, LOW);
          digitalWrite(LEFT_REV, LOW);
          mode=stopped;
         if(debug)
            Serial.print ("STOPPED-30seconds ");
         // start by tapping front bumper
         hitMillis=millis();    // time of LED blink
         while(digitalRead(frontBumper)){
             if (millis() - hitMillis > 330){
                 LED_on = !LED_on;
                 if (LED_on)
                   digitalWrite(LED, HIGH);
                 else
                   digitalWrite(LED, LOW);
                 hitMillis=millis();    // time of LED blink
             }
         }
         digitalWrite(LED, HIGH);
         delay(2500);    //wait 3 seconds before moving
     if(!debug){  // turn off motors in debug mode
         analogWrite(RIGHT_FWD, 75);
         analogWrite(LEFT_FWD, 75);
     }
          mode=forward;   
          hit_stop=0;
          hits = 0;
          digitalWrite(LED, HIGH);
          delay(330);
          digitalWrite(LED, LOW);
          delay(330);

  }
  

}



